import { LitElement } from 'lit';
import { EditorView } from '@codemirror/view';
import { type EditorSelection, Transaction, type Extension, type Text, type ChangeSpec } from '@codemirror/state';
/**
 * An event emmitted for all CodeMirror transactions.
 *
 * Calling `preventDefault()` on this event will prevent the transaction from
 * being applied to the editor.
 */
export declare class TransactionEvent extends Event {
    static readonly eventName = "codemirror-transaction";
    readonly transaction: Transaction;
    target: CodeMirrorEditor;
    constructor(transaction: Transaction);
}
/**
 * An event emmitted for all CodeMirror document changes. This event is emitted
 * after the transaction event.
 *
 * Calling `preventDefault()` on this event will prevent the transaction from
 * being applied to the editor.
 */
export declare class DocumentChangeEvent extends Event {
    static readonly eventName = "codemirror-document-change";
    readonly transaction: Transaction;
    changes: Array<Change>;
    target: CodeMirrorEditor;
    constructor(transaction: Transaction, changes: Array<Change>);
}
export type Change = {
    fromA: number;
    toA: number;
    fromB: number;
    toB: number;
    inserted: Text;
};
/**
 * An event emmitted for all CodeMirror selection changes. This event is emitted
 * after the transaction event.
 *
 * Calling `preventDefault()` on this event will prevent the transaction from
 * being applied to the editor.
 */
export declare class SelectionChangeEvent extends Event {
    static readonly eventName = "codemirror-selection-change";
    readonly transaction: Transaction;
    readonly selection: EditorSelection;
    target: CodeMirrorEditor;
    constructor(transaction: Transaction, selection: EditorSelection);
}
declare global {
    interface HTMLElementTagNameMap {
        'cm-editor': CodeMirrorEditor;
    }
}
export declare class CodeMirrorEditor extends LitElement {
    #private;
    static styles: import("lit").CSSResult;
    editorView: EditorView;
    get value(): string | undefined;
    set value(v: string | undefined);
    constructor();
    render(): HTMLElement;
    createRenderRoot(): Element | ShadowRoot;
    changeDocument(change: ChangeSpec): void;
    addExtensions(extensions: Array<Extension>): void;
    removeExtensions(extensions: Array<Extension>): void;
    private _createEditorView;
}
//# sourceMappingURL=cm-editor.d.ts.map