var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, LitElement } from 'lit';
import { customElement, property } from 'lit/decorators.js';
import { ContextProvider } from '@lit-labs/context';
import { EditorView, keymap, highlightSpecialChars, drawSelection, highlightActiveLine, dropCursor, rectangularSelection, crosshairCursor, lineNumbers, highlightActiveLineGutter, } from '@codemirror/view';
import { Compartment, EditorState, Transaction, } from '@codemirror/state';
import { defaultHighlightStyle, syntaxHighlighting, indentOnInput, bracketMatching, foldGutter, foldKeymap, } from '@codemirror/language';
import { defaultKeymap, history, historyKeymap } from '@codemirror/commands';
import { searchKeymap, highlightSelectionMatches } from '@codemirror/search';
import { autocompletion, completionKeymap, closeBrackets, closeBracketsKeymap, } from '@codemirror/autocomplete';
import { lintKeymap } from '@codemirror/lint';
import { extensionsContext } from './extension-host-context.js';
/**
 * An event emmitted for all CodeMirror transactions.
 *
 * Calling `preventDefault()` on this event will prevent the transaction from
 * being applied to the editor.
 */
class TransactionEvent extends Event {
    static { this.eventName = 'codemirror-transaction'; }
    constructor(transaction) {
        super(TransactionEvent.eventName, { bubbles: true });
        this.transaction = transaction;
    }
}
export { TransactionEvent };
/**
 * An event emmitted for all CodeMirror document changes. This event is emitted
 * after the transaction event.
 *
 * Calling `preventDefault()` on this event will prevent the transaction from
 * being applied to the editor.
 */
class DocumentChangeEvent extends Event {
    static { this.eventName = 'codemirror-document-change'; }
    constructor(transaction, changes) {
        super(DocumentChangeEvent.eventName, { bubbles: true });
        this.transaction = transaction;
        this.changes = changes;
    }
}
export { DocumentChangeEvent };
/**
 * An event emmitted for all CodeMirror selection changes. This event is emitted
 * after the transaction event.
 *
 * Calling `preventDefault()` on this event will prevent the transaction from
 * being applied to the editor.
 */
class SelectionChangeEvent extends Event {
    static { this.eventName = 'codemirror-selection-change'; }
    constructor(transaction, selection) {
        super(SelectionChangeEvent.eventName, { bubbles: true });
        this.transaction = transaction;
        this.selection = selection;
    }
}
export { SelectionChangeEvent };
let CodeMirrorEditor = class CodeMirrorEditor extends LitElement {
    static { this.styles = css `
    :host {
      display: block;
      /* TODO: grab from theme */
      border: solid 1px rgb(221, 221, 221);
    }
  `; }
    get value() {
        return this.editorView.state.doc.toString();
    }
    set value(v) {
        this.editorView.dispatch({
            changes: [{ from: 0, to: this.editorView.state.doc.length, insert: v }],
        });
    }
    constructor() {
        super();
        this._createEditorView();
        new ContextProvider(this, { context: extensionsContext, initialValue: this });
    }
    #addedExtensions = new Set();
    #addedExtensionCompartment = new Compartment();
    render() {
        return this.editorView.dom;
    }
    createRenderRoot() {
        const root = super.createRenderRoot();
        this.editorView.setRoot(root);
        return root;
    }
    changeDocument(change) {
        this.editorView.dispatch({
            changes: [change],
        });
    }
    addExtensions(extensions) {
        for (const extension of extensions) {
            this.#addedExtensions.add(extension);
        }
        console;
        const effect = this.#addedExtensionCompartment.reconfigure([
            ...this.#addedExtensions,
        ]);
        this.editorView?.dispatch({ effects: [effect] });
    }
    removeExtensions(extensions) {
        for (const extension of extensions) {
            this.#addedExtensions.delete(extension);
        }
        const effect = this.#addedExtensionCompartment.reconfigure([
            ...this.#addedExtensions,
        ]);
        this.editorView?.dispatch({ effects: [effect] });
    }
    _createEditorView() {
        const view = (this.editorView = new EditorView({
            dispatch: (t) => {
                let notPrevented = this.dispatchEvent(new TransactionEvent(t));
                if (t.docChanged) {
                    const changes = [];
                    t.changes.iterChanges((fromA, toA, fromB, toB, inserted) => {
                        changes.push({ fromA, toA, fromB, toB, inserted });
                    });
                    notPrevented &&= this.dispatchEvent(new DocumentChangeEvent(t, changes));
                }
                if (t.selection) {
                    notPrevented &&= this.dispatchEvent(new SelectionChangeEvent(t, t.selection));
                }
                if (notPrevented) {
                    view.update([t]);
                }
            },
            extensions: [
                lineNumbers(),
                highlightActiveLineGutter(),
                highlightSpecialChars(),
                history(),
                foldGutter(),
                drawSelection(),
                dropCursor(),
                EditorState.allowMultipleSelections.of(true),
                indentOnInput(),
                syntaxHighlighting(defaultHighlightStyle, { fallback: true }),
                bracketMatching(),
                closeBrackets(),
                autocompletion(),
                rectangularSelection(),
                crosshairCursor(),
                highlightActiveLine(),
                highlightSelectionMatches(),
                keymap.of([
                    ...closeBracketsKeymap,
                    ...defaultKeymap,
                    ...searchKeymap,
                    ...historyKeymap,
                    ...foldKeymap,
                    ...completionKeymap,
                    ...lintKeymap,
                ]),
                this.#addedExtensionCompartment.of([]),
            ],
        }));
    }
};
__decorate([
    property({ attribute: false })
], CodeMirrorEditor.prototype, "editorView", void 0);
__decorate([
    property()
], CodeMirrorEditor.prototype, "value", null);
CodeMirrorEditor = __decorate([
    customElement('cm-editor')
], CodeMirrorEditor);
export { CodeMirrorEditor };
//# sourceMappingURL=cm-editor.js.map